/*
 * gSXprog by Eric Smith, based on sxprog by Loren Blaney
 *
 * Copyright 2001-2003 Eric Smith <eric@brouhaha.com>
 *
 * $Id: sxkey_old.c,v 1.4 2003/11/18 13:59:42 garlick Exp $
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <termios.h>            /* to get speed (B19200 etc.) */
#include <assert.h>
#include <libgen.h>
#include <getopt.h>

#include "gsxprog.h"
#include "sxkey.h"
#include "hex_file.h"
#include "sxkey_old.h"


static u16 _convert_time(u16 t)
{
    return (~((t * 11) / 8));
}


static u16 _convert_data(u16 d)
{
    return ((d << 1) | 0x8001);
}


static boolean _read_word(int fd, int ms_timeout, u16 * word)
{
    boolean r;
    r = sx_send_byte(fd, SXKEY_OLD_CMD_READ);
    if (!r)
        return (r);
    r = sx_get_word(fd, ms_timeout, word);
    return (r);
}


static boolean _check_error(int fd, int ms_timeout)
{
    boolean r;
    u8 c;

    r = sx_get_byte(fd, ms_timeout, &c);
    if (!r)
        return (r);
    if (c != 0x00) {
        fprintf(stderr, "%s: SX-Key returned error code %02x\n", progname,
                c);
        return (0);
    }
    return (1);
}


static void old_fini(sxkey_t * key)
{
    int fd = key->fd;

    sx_serial_close(fd);
}


static boolean old_read(sxkey_t * key, sx_t * sx)
{
    int fd = key->fd;
    int i, size_a, size_c, size_rom;
    boolean r;

    r = _read_word(fd, SXKEY_OLD_READ_TIMEOUT, &sx->device);
    if (!r)
        return (r);

    r = _read_word(fd, SXKEY_OLD_READ_TIMEOUT, &sx->rom[FUSEX_LOC]);
    if (!r)
        return (r);

    r = _read_word(fd, SXKEY_OLD_READ_TIMEOUT, &sx->rom[FUSE_LOC]);
    if (!r)
        return (r);

    /*
     * Set SizeROM to the smaller of the configured size (from FUSEX) or the
     * actual size (from DEVICE) (an erased chip will look like it's 4096 words)
     */

    size_a = (sx->device & 0x003) << 10;
    size_c = (sx->rom[FUSEX_LOC] & 0x003) << 10;
    size_rom = size_c;
    if (size_a < size_c)
        size_rom = size_a;

    for (i = 0; i < size_rom; i++) {
        r = _read_word(fd, SXKEY_OLD_READ_TIMEOUT, &sx->rom[i]);
        if (!r)
            return (r);
    }

    for (i = 0; i < SX_ID_SIZE; i++) {
        r = _read_word(fd, SXKEY_OLD_READ_TIMEOUT, &sx->rom[ID_LOC + i]);
        if (!r)
            return (r);
    }

    return (r);
}


static boolean old_erase(sxkey_t * key)
{
    int fd = key->fd;
    boolean r;

    r = sx_send_byte(fd, SXKEY_OLD_CMD_ERASE);
    if (!r)
        return (r);

    r = sx_send_word(fd, _convert_time(SX_OLD_ERASE_MS));
    if (!r)
        return (r);

    r = sx_send_word(fd, 0xffff);
    if (!r)
        return (r);

    r = _check_error(fd, SX_OLD_ERASE_MS + SXKEY_OLD_ERASE_TIMEOUT);

    r = sx_send_byte(fd, '\0');
    if (!r)
        return (r);

    return (r);
}


static boolean old_prog(sxkey_t * key, sx_t * sx, boolean trim, boolean idskip)
{
    int fd = key->fd;
    boolean r;
    int i;

    if (trim) {
        fprintf(stderr, "%s: clock trimming unimplemented for older SX-Keys\n",
                progname);
    }
    if (idskip) {
        fprintf(stderr, "%s: idskip unimplemented for older SX-Keys\n", progname);
    }

    /* should read DEVICE (and FUSEX?) here */

    r = sx_send_byte(fd, SXKEY_OLD_CMD_ERASE);
    if (!r)
        return (r);

    sx_ms_delay(10);

    r = sx_send_word(fd, 0xffff);
    if (!r)
        return (r);

    r = sx_send_word(fd, _convert_time(SX_OLD_PROG_MS));
    if (!r)
        return (r);

    r = _check_error(fd, 900);
    if (!r)
        return (r);

    r = _check_error(fd, SX_OLD_PROG_MS + SXKEY_OLD_PROG_SETUP_TIMEOUT);
    if (!r)
        return (r);

    r = sx_send_word(fd, _convert_data(sx->rom[FUSEX_LOC]));
    if (!r)
        return (r);
    r = _check_error(fd, SX_OLD_PROG_MS + SXKEY_OLD_PROG_TIMEOUT);
    if (!r)
        return (r);

    r = sx_send_word(fd, _convert_data(sx->rom[FUSE_LOC]));
    if (!r)
        return (r);
    r = _check_error(fd, SX_OLD_PROG_MS + SXKEY_OLD_PROG_TIMEOUT);
    if (!r)
        return (r);

    for (i = 0; i < SX_MAX_ROM_SIZE; i++) {
        r = sx_send_word(fd, _convert_data(sx->rom[i]));
        if (!r)
            return (r);
        r = _check_error(fd, SX_OLD_PROG_MS + SXKEY_OLD_PROG_TIMEOUT);
        if (!r)
            return (r);
    }

    for (i = 0; i < SX_ID_SIZE; i++) {
        r = sx_send_word(fd, _convert_data(sx->rom[ID_LOC + i]));
        if (!r)
            return (r);
        r = _check_error(fd, SX_OLD_PROG_MS + SXKEY_OLD_PROG_TIMEOUT);
        if (!r)
            return (r);
    }

    r = sx_send_byte(fd, '\0');
    if (!r)
        return (r);

    return (r);
}

boolean old_init(char *dev, speed_t baud, sxkey_t * key)
{
    boolean r;
    int fd;

    r = sx_serial_open(dev, baud, &fd);
    if (!r)
        return (0);

    r = sx_serial_flush(fd);
    if (!r)
        return (0);

    r = sx_send_byte(fd, '\0');
    if (!r)
        return (0);
    r = sx_send_str(fd, "SX-Key");
    if (!r)
        return (0);
    r = sx_expect_str(fd, SXKEY_OLD_PROLOG_TIMEOUT, "SX-Key");
    if (!r)
        return (0);

    if (key) {
        key->cmd_prog = old_prog;
        key->cmd_erase = old_erase;
        key->cmd_read = old_read;
        key->fini = old_fini;
        key->fd = fd;
        key->speed = baud;
        key->device = dev;
    }
    return (1);
}

/*
 * em:Local Variables: 
 * em:indent-tabs-mode:nil
 * em:c-basic-offset:4
 * em:End:
 * vi:tabstop=4 shiftwidth=4 expandtab
 */
